--------------------------------------------------------------------------------
-- Company: <Name>
--
-- File: cntr_ctrl.vhd
-- File history:
--      <Revision number>: <Date>: <Comments>
--      <Revision number>: <Date>: <Comments>
--      <Revision number>: <Date>: <Comments>
--
-- Description: 
--
-- <Description here>
--
-- Targeted device: <Family::PolarFire> <Die::MPF300T_ES> <Package::FCG1152>
-- Author: <Name>
--
--------------------------------------------------------------------------------

library IEEE;

use IEEE.std_logic_1164.all;
use IEEE.STD_LOGIC_arith.all;
use IEEE.STD_LOGIC_UNSIGNED.all;

entity cntr_ctrl is
port (
    nReset   : in  std_logic;
    clk      : in  std_logic;
	start    : in  std_logic;
    done_flag: out std_logic;
	done_int : out std_logic;
    enable   : out std_logic;
    cntr_rst : out std_logic
);
end cntr_ctrl;

architecture architecture_cntr_ctrl of cntr_ctrl is

    type states_type is (idle, reset, running, done);
    signal state : states_type;

    signal   cntr     : std_logic_vector(31 downto 0);
    constant cntr_max : std_logic_vector(31 downto 0) := conv_std_logic_vector(80000000-1, 32);


    signal start_i : std_logic_vector(1 downto 0);

begin

    start_sync_P : process(clk, nReset)
    begin
        if(nReset = '0') then
            start_i <= (others => '0');
        elsif(clk'event and clk = '1') then
            start_i <= start_i(0) & start;
        end if;
    end process start_sync_P;

    FSM_P : process(clk, nReset)
    begin
        if(nReset = '0') then
            state <= idle;
        elsif(clk'event and clk = '1') then
            case state is
                when idle =>
                    if(start_i = "01") then
                        state <= reset;
                    end if;
                when reset =>
                    state <= running;
                when running =>
                    if(cntr = cntr_max) then
                        state <= done;
                    end if;
                when done =>
                    state <= idle;
                when others => state <= idle;
            end case;
        end if;
    end process FSM_P;

    cntP : process(clk, nReset)
    begin
        if(nReset = '0') then
            cntr <= (others => '0');
        elsif(clk'event and clk = '1') then
            if(state = running) then
                cntr <= cntr + 1;
            elsif(state = reset) then
                cntr <= (others => '0');
            end if;
        end if;
    end process cntP;

    enable   <= '1' when (state = running) else '0';
    cntr_rst <= '1' when (state = reset) else '0';

    done_int <= '1' when (state = done) else '0';
    done_flag<= '1' when (state = idle) else '0';

end architecture_cntr_ctrl;
